@@ -23,7 +23,7 @@
 #   This specific module is licensed under the Perl Artistic License.
 #
 #
-#   $Id: File.pm,v 1.21 2005/10/21 21:42:56 fredo Exp $
+#   $Id: File.pm,v 2.4 2007/03/10 11:10:28 areibens Exp $
 #
 #=======================================================================
 package PDF::API2::Basic::PDF::File;
@@ -201,8 +201,7 @@ use PDF::API2::Basic::PDF::Null;
 
 no warnings qw[ deprecated recursion uninitialized ];
 
-    # $Revisioning: 0.40_11 on Mon Oct  6 18:39:40 2003 $
-    ( $VERSION ) = '0.40_10';
+    ( $VERSION ) = sprintf '%i.%03i', split(/\./,('$Revision: 2.4 $' =~ /Revision: (\S+)\s/)[0]); # $Date: 2007/03/10 11:10:28 $
 
 #IMPORTED INTO PDF::API2
 
@@ -442,15 +441,9 @@ sub append_file
     # hack to upgrade pdf-version number to support
     # requested features in higher versions that
     # the pdf was originally created.
-    $fh->seek(7,0);
-    $fh->read($buf, 3);
-    $buf=~s/[^\d]+$//g;
-    $ver=$self->{' version'} || 2;
-    if($buf < $ver) {
-    ##  print STDERR "files version was 1.$buf upgraded to 1.$ver.\n";
-        $fh->seek(0,0);
-        $fh->print("%PDF-1.$ver\n");
-    }
+    $ver=$self->{' version'} || 4;
+    $fh->seek(0,0);
+    $fh->print("%PDF-1.$ver\n");
 
     $tdict = PDFDict();
     $tdict->{'Prev'} = PDFNum($self->{' loc'});
@@ -514,7 +507,7 @@ sub create_file
 
     $self->{' OUTFILE'} = $fh;
     $fh->print('%PDF-1.' . ($self->{' version'} || '2') . "\n");
-    $fh->print("%쏢\n");              # and some binary stuff in a comment
+    $fh->print("%\xC7\xEC\xF3\xA2\n");   # and some binary stuff in a comment
     $self;
 }
 
@@ -573,14 +566,14 @@ sub readval
     my ($fh) = $self->{' INFILE'};
     my ($res, $key, $value, $k);
 
-    #$str = update($fh, $str);
-    updateRef($fh, \$str);
+    $str = update($fh, $str);
+    ## updateRef($fh, \$str);
 
     if ($str =~ m/^<</so)
     {
         $str = substr ($str, 2);
-        #$str = update($fh, $str);
-        updateRef($fh, \$str);
+        $str = update($fh, $str);
+        ## updateRef($fh, \$str);
         $res = PDFDict();
 
         while ($str !~ m/^>>/o) {
@@ -602,12 +595,12 @@ sub readval
                 ($value, $str) = $self->readval($str, %opts);
                 $res->{null} = $value;
             }
-            #$str = update($fh, $str); # thanks gareth.jones@stud.man.ac.uk
-            updateRef($fh, \$str); # thanks gareth.jones@stud.man.ac.uk
+            $str = update($fh, $str); # thanks gareth.jones@stud.man.ac.uk
+            ## updateRef($fh, \$str); # thanks gareth.jones@stud.man.ac.uk
         }
         $str =~ s/^>>//o;
-        #$str = update($fh, $str);
-        updateRef($fh, \$str);
+        $str = update($fh, $str);
+        ## updateRef($fh, \$str);
     # streams can't be followed by a lone carriage-return.
     # fredo: yes they can !!! -- use the MacOS Luke.
         if (($str =~ s/^stream(?:(?:\015\012)|\012|\015)//o)
@@ -628,8 +621,8 @@ sub readval
                 $value .= substr($str, 0, $k);
                 $res->{' stream'} = $value;
                 $res->{' nofilt'} = 1;
-                #$str = update($fh, $str, 1);  # tell update we are in-stream and only need an endstream 
-                updateRef($fh, \$str, 1);  # tell update we are in-stream and only need an endstream 
+                $str = update($fh, $str, 1);  # tell update we are in-stream and only need an endstream 
+                ## updateRef($fh, \$str, 1);  # tell update we are in-stream and only need an endstream 
                 #$str =~ s/^endstream//o; # we cannot regexpr here since we need the first endstream only
                 # so we do the following:
                 my $wh = index($str,'endstream');
@@ -674,8 +667,8 @@ sub readval
             $self->add_obj($res, $k, $value);
             $res->{' realised'} = 1;
         }
-        #$str = update($fh, $str);       # thanks to kundrat@kundrat.sk
-        updateRef($fh, \$str);       # thanks to kundrat@kundrat.sk
+        $str = update($fh, $str);       # thanks to kundrat@kundrat.sk
+        ## updateRef($fh, \$str);       # thanks to kundrat@kundrat.sk
         $str =~ s/^endobj//o;
     } elsif ($str =~ m|^/($reg_char+)|so)        # name
     {
@@ -683,46 +676,54 @@ sub readval
         $value = $1;
         $str =~ s|^/($reg_char+)||so;
         $res = PDF::API2::Basic::PDF::Name->from_pdf($value, $self);
-    } elsif ($str =~ m/^\(/o)  # literal string
+    } 
+    elsif ($str =~ m/^\(/o)  # literal string
     {
         $str =~ s/^\(//o;
-    # We now need to find an unbalanced, unescaped right-paren.
-    # This can't be done with regexps.
-    my ($value) = "";
-    # The current level of nesting, when this reaches 0 we have finished.
-    my ($nested) = 1;
-    while (1) {
-        # Remove everything up to the first (possibly escaped) paren.
-        $str =~ /^((?:[^\\()]|\\[^()])*)(.*)/so;
-        $value .= $1;
-        $str = $2;
-
-        if ($str =~ /^(\\[()])/o) {
-        # An escaped paren.  This would be tricky to do with
-        # the regexp above (it's very difficult to be certain
-        # that all cases are covered so I think it's better to
-        # deal with them explicitly).
-        $str = substr ($str, 2);
-        $value = $value . $1;
-        } elsif ($str =~ /^\)/o) {
-        # Right paren
-        $nested--;
-        $str = substr ($str, 1);
-        if ($nested == 0)
-            { last; }
-        $value = $value . ')';
-        } elsif ($str =~ /^\(/o) {
-        # Left paren
-        $nested++;
-        $str = substr ($str, 1);
-        $value = $value . '(';
-        } else {
-        # No parens, we must read more.  We don't use update
-        # because we don't want to remove whitespace or
-        # comments.
-        $fh->read($str, 255, length($str)) or die "Unterminated string.";
+        # We now need to find an unbalanced, unescaped right-paren.
+        # This can't be done with regexps.
+        my ($value) = "";
+        # The current level of nesting, when this reaches 0 we have finished.
+        my ($nested) = 1;
+        while (1) {
+            # Remove everything up to the first (possibly escaped) paren.
+            $str =~ /^((?:[^\\()]|\\[^()])*)(.*)/so;
+            $value .= $1;
+            $str = $2;
+    
+            if ($str =~ /^(\\[()])/o) 
+            {
+                # An escaped paren.  This would be tricky to do with
+                # the regexp above (it's very difficult to be certain
+                # that all cases are covered so I think it's better to
+                # deal with them explicitly).
+                $str = substr ($str, 2);
+                $value = $value . $1;
+            } 
+            elsif ($str =~ /^\)/o) 
+            {
+                # Right paren
+                $nested--;
+                $str = substr ($str, 1);
+                if ($nested == 0)
+                    { last; }
+                $value = $value . ')';
+            } 
+            elsif ($str =~ /^\(/o) 
+            {
+                # Left paren
+                $nested++;
+                $str = substr ($str, 1);
+                $value = $value . '(';
+            } 
+            else 
+            {
+                # No parens, we must read more.  We don't use update
+                # because we don't want to remove whitespace or
+                # comments.
+                $fh->read($str, 255, length($str)) or die "Unterminated string.";
+            }
         }
-    }
 
         $res = PDF::API2::Basic::PDF::String->from_pdf($value);
     } 
@@ -736,15 +737,15 @@ sub readval
     elsif ($str =~ m/^\[/o)                                      # array
     {
         $str =~ s/^\[//o;
-        #$str = update($fh, $str);
-        updateRef($fh, \$str);
+        $str = update($fh, $str);
+        ## updateRef($fh, \$str);
         $res = PDFArray();
         while ($str !~ m/^\]/o)
         {
             ($value, $str) = $self->readval($str, %opts);
             $res->add_elements($value);
-            #$str = update($fh, $str);   # str might just be exhausted!
-            updateRef($fh, \$str);   # str might just be exhausted!
+            $str = update($fh, $str);   # str might just be exhausted!
+            ## updateRef($fh, \$str);   # str might just be exhausted!
         }
         $str =~ s/^\]//o;
     } 
@@ -1246,10 +1247,12 @@ sub readxrtr
         }
     }
 
-    if ($buf !~ /^trailer$cr/oi)
+#    if ($buf !~ /^trailer$cr/oi)
+    if ($buf !~ /^trailer\b/oi)
     { die "Malformed trailer in PDF file $self->{' fname'} at " . ($fh->tell - length($buf)); }
 
-    $buf =~ s/^trailer$cr//oi;
+#    $buf =~ s/^trailer$cr//oi;
+    $buf =~ s/^trailer\b//oi;
 
     ($tdict, $buf) = $self->readval($buf);
     $tdict->{' loc'} = $xpos;
